@@ -13,7 +13,7 @@
 ##' \itemize{
 ##' \item{Chunker: force=rescue}
 ##' \item{G2P: embed=maus}
-##' \item{Pho2Syl: wsync=no}
+##' \item{Pho2Syl: wsync=yes}
 ##' \item{MAUS: USETRN=[true if Chunker was called or transcription is a segment tier, false otherwise]}
 ##' }
 ##' If you wish to change parameters, you must use the individual runBASwebservices functions. This will also allow
@@ -40,6 +40,8 @@
 ##' @param sylAttributeDefinitionName attribute name for syllable segmentation
 ##' @param canoSylAttributeDefinitionName attribute name for syllabified canonical pronunciations of words
 ##'
+##' @param patience If a web service call fails, it is repeated a further n times, with n being the value of patience.
+##' Must be set to a value between 0 and 3.
 ##' @param verbose Display progress bars and other information
 ##' @param resume If a previous call to this function has failed (and you think you have fixed the issue
 ##' that caused the error), you can set resume=TRUE to recover any progress made up to that point. This
@@ -57,9 +59,12 @@ runBASwebservice_all <- function(handle,
                                  canoSylAttributeDefinitionName = "KAS",
                                  chunkAttributeDefinitionName = "TRN",
                                  
+                                 patience = 0,
                                  resume = FALSE,
                                  verbose = TRUE)
 {
+  check_emuDBhandle(handle)
+  
   func = "all"
   transcriptionLevel = get_levelNameForAttributeName(handle, transcriptionAttributeDefinitionName)
   
@@ -101,7 +106,8 @@ runBASwebservice_all <- function(handle,
     verbose = verbose,
     resume = resume,
     params = list(),
-    func = func
+    func = func,
+    patience = patience
   )
   
   bas_run_g2p_for_pronunciation_dbi(
@@ -112,7 +118,8 @@ runBASwebservice_all <- function(handle,
     verbose = verbose,
     resume = resume,
     params = list(embed = "maus"),
-    func = func
+    func = func,
+    patience = patience
   )
   
   
@@ -131,7 +138,8 @@ runBASwebservice_all <- function(handle,
       language = language,
       oldBasePath = oldBasePath,
       perspective = "default",
-      func = func
+      func = func,
+      patience = patience
     )
   }
   
@@ -147,7 +155,8 @@ runBASwebservice_all <- function(handle,
     oldBasePath = oldBasePath,
     perspective = "default",
     turnChunkLevelIntoItemLevel = T,
-    func = func
+    func = func,
+    patience = patience
   )
   
   bas_run_minni_dbi(
@@ -160,7 +169,8 @@ runBASwebservice_all <- function(handle,
     params = list(),
     oldBasePath = oldBasePath,
     perspective = "default",
-    func = func
+    func = func,
+    patience = patience
   )
   
   bas_run_pho2syl_canonical_dbi(
@@ -171,7 +181,8 @@ runBASwebservice_all <- function(handle,
     verbose = verbose,
     params = list(),
     resume = resume,
-    func = func
+    func = func,
+    patience = patience
   )
   
   orthoLevel = orthoAttributeDefinitionName
@@ -185,7 +196,8 @@ runBASwebservice_all <- function(handle,
     resume = resume,
     params = list(wsync = "yes"),
     verbose = verbose,
-    func = func
+    func = func,
+    patience = patience
   )
   
   
@@ -235,7 +247,7 @@ runBASwebservice_all <- function(handle,
 ##' @param chunkLevel if you have a chunk segmentation level, you can provide it to improve the speed and accuracy
 ##' of MAUS. The chunk segmentation level must be a segment level, and it must link to the level of canoAttributeDefinitionName.
 ##' @param turnChunkLevelIntoItemLevel if TRUE, and if a chunk level is provided, the chunk level is converted into an ITEM level after segmentation
-##' @param params named list of parameters to be passed on to the webservice. It is your own reponsibility to
+##' @param params named list of parameters to be passed on to the webservice. It is your own responsibility to
 ##' ensure that these parameters are compatible with the webservice API
 ##' (see \url{https://clarin.phonetik.uni-muenchen.de/BASWebServices/services/help}).
 ##' Some options accepted by the API (e.g. output format) cannot be set when calling a webservice from within emuR,
@@ -257,9 +269,12 @@ runBASwebservice_maus <- function(handle,
                                   params = NULL,
                                   
                                   perspective = "default",
+                                  patience = 0,
                                   resume = FALSE,
                                   verbose = TRUE)
 {
+  check_emuDBhandle(handle)
+  
   func = "maus"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -276,7 +291,8 @@ runBASwebservice_maus <- function(handle,
     oldBasePath = oldBasePath,
     perspective = perspective,
     turnChunkLevelIntoItemLevel = turnChunkLevelIntoItemLevel,
-    func = func
+    func = func,
+    patience = patience
   )
   
   handle = bas_clear(handle, oldBasePath, func)
@@ -313,9 +329,12 @@ runBASwebservice_g2pForTokenization <- function(handle,
                                                 
                                                 params = list(),
                                                 
+                                                patience = 0,
                                                 resume = FALSE,
                                                 verbose = TRUE)
 {
+  check_emuDBhandle(handle)
+  
   func = "g2p_tokenization"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -328,7 +347,8 @@ runBASwebservice_g2pForTokenization <- function(handle,
     verbose = verbose,
     resume = resume,
     params = params,
-    func = func
+    func = func,
+    patience = patience
   )
   
   handle = bas_clear(handle, oldBasePath, func)
@@ -365,9 +385,11 @@ runBASwebservice_g2pForPronunciation <- function(handle,
                                                  
                                                  params = list(embed = "maus"),
                                                  
+                                                 patience = 0,
                                                  resume = FALSE,
                                                  verbose = TRUE)
 {
+  check_emuDBhandle(handle)
   func = "g2p_pronunciation"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -380,7 +402,8 @@ runBASwebservice_g2pForPronunciation <- function(handle,
     verbose = verbose,
     resume = resume,
     params = params,
-    func = func
+    func = func,
+    patience = patience
   )
   
   handle = bas_clear(handle, oldBasePath, func)
@@ -434,9 +457,11 @@ runBASwebservice_chunker <- function(handle,
                                      params = list(force = "rescue"),
                                      
                                      perspective = "default",
+                                     patience = 0,
                                      resume = FALSE,
                                      verbose = TRUE)
 {
+  check_emuDBhandle(handle)
   func = "chunker"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -453,7 +478,8 @@ runBASwebservice_chunker <- function(handle,
     perspective = perspective,
     resume = resume,
     rootLevel = rootLevel,
-    func = func
+    func = func,
+    patience = patience
   )
   
   handle = bas_clear(handle, oldBasePath, func)
@@ -495,9 +521,11 @@ runBASwebservice_minni <- function(handle,
                                    params = list(),
                                    
                                    perspective = "default",
+                                   patience = 0,
                                    resume = FALSE,
                                    verbose = TRUE)
 {
+  check_emuDBhandle(handle)
   func = "minni"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -512,7 +540,8 @@ runBASwebservice_minni <- function(handle,
     params = params,
     oldBasePath = oldBasePath,
     perspective = perspective,
-    func = func
+    func = func,
+    patience = patience
   )
   
   handle = bas_clear(handle, oldBasePath, func)
@@ -546,9 +575,11 @@ runBASwebservice_pho2sylCanonical <- function(handle,
                                               
                                               params = list(),
                                               
+                                              patience = 0,
                                               resume = FALSE,
                                               verbose = TRUE)
 {
+  check_emuDBhandle(handle)
   func = "pho2syl_canonical"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -561,7 +592,8 @@ runBASwebservice_pho2sylCanonical <- function(handle,
     canoSylAttributeDefinitionName = canoSylAttributeDefinitionName,
     resume = resume,
     params = params,
-    func = func
+    func = func,
+    patience = patience
   )
   
   
@@ -604,9 +636,11 @@ runBASwebservice_pho2sylSegmental <- function(handle,
                                               params = list(wsync = "yes"),
                                               
                                               perspective = "default",
+                                              patience = 0,
                                               resume = FALSE,
                                               verbose = TRUE)
 {
+  check_emuDBhandle(handle)
   func = "pho2syl_segmental"
   oldBasePath = handle$basePath
   handle = bas_prepare(handle, resume, verbose, func)
@@ -620,7 +654,8 @@ runBASwebservice_pho2sylSegmental <- function(handle,
     superLevel = superLevel,
     resume = resume,
     params = params,
-    func = func
+    func = func,
+    patience = patience
   )
   
   
